#!/usr/bin/env python3
"""
Network Interface Profiler
Parameters:
    - IP: Interface IP to profile
Additional parameters in comments:
    - duration: Profile duration in seconds
"""
import psutil
import time
import argparse
from datetime import datetime
import json
from collections import defaultdict

class NetworkProfiler:
    def __init__(self, interface_ip, duration=60):
        self.interface_ip = interface_ip
        self.duration = duration
        self.stats = defaultdict(list)
        self.interface_name = self._get_interface_name()
        
    def _get_interface_name(self):
        """Get interface name from IP address"""
        for iface, addrs in psutil.net_if_addrs().items():
            for addr in addrs:
                if addr.address == self.interface_ip:
                    return iface
        raise ValueError(f"No interface found with IP {self.interface_ip}")
        
    def collect_interface_info(self):
        """Collect static interface information"""
        addrs = psutil.net_if_addrs()[self.interface_name]
        stats = psutil.net_if_stats()[self.interface_name]
        
        info = {
            "name": self.interface_name,
            "addresses": [],
            "speed": stats.speed,
            "mtu": stats.mtu,
            "is_up": stats.isup,
            "duplex": stats.duplex,
        }
        
        for addr in addrs:
            addr_info = {
                "family": str(addr.family),
                "address": addr.address,
            }
            if addr.netmask:
                addr_info["netmask"] = addr.netmask
            if hasattr(addr, 'broadcast') and addr.broadcast:
                addr_info["broadcast"] = addr.broadcast
            info["addresses"].append(addr_info)
            
        return info
        
    def collect_metrics(self):
        """Collect network metrics over time"""
        start_time = time.time()
        last_io = psutil.net_io_counters(pernic=True)[self.interface_name]
        last_time = start_time
        
        print(f"Collecting network metrics for {self.duration} seconds...")
        
        while time.time() - start_time < self.duration:
            time.sleep(1)
            current_time = time.time()
            current_io = psutil.net_io_counters(pernic=True)[self.interface_name]
            
            # Calculate rates
            time_delta = current_time - last_time
            bytes_sent = current_io.bytes_sent - last_io.bytes_sent
            bytes_recv = current_io.bytes_recv - last_io.bytes_recv
            packets_sent = current_io.packets_sent - last_io.packets_sent
            packets_recv = current_io.packets_recv - last_io.packets_recv
            
            # Store metrics
            self.stats["timestamp"].append(current_time)
            self.stats["bytes_sent_per_sec"].append(bytes_sent / time_delta)
            self.stats["bytes_recv_per_sec"].append(bytes_recv / time_delta)
            self.stats["packets_sent_per_sec"].append(packets_sent / time_delta)
            self.stats["packets_recv_per_sec"].append(packets_recv / time_delta)
            self.stats["errors_in"].append(current_io.errin)
            self.stats["errors_out"].append(current_io.errout)
            self.stats["drops_in"].append(current_io.dropin)
            self.stats["drops_out"].append(current_io.dropout)
            
            last_io = current_io
            last_time = current_time
            
            # Print current stats
            print(f"\rThroughput - Upload: {bytes_sent/time_delta/1024:.2f} KB/s, "
                  f"Download: {bytes_recv/time_delta/1024:.2f} KB/s", end="")
            
    def generate_report(self):
        """Generate a detailed report of the network profile"""
        report = {
            "timestamp": datetime.now().isoformat(),
            "interface_info": self.collect_interface_info(),
            "metrics": {
                "duration": self.duration,
                "samples": len(self.stats["timestamp"]),
                "throughput": {
                    "upload": {
                        "avg": sum(self.stats["bytes_sent_per_sec"]) / len(self.stats["bytes_sent_per_sec"]) / 1024,
                        "max": max(self.stats["bytes_sent_per_sec"]) / 1024,
                        "min": min(self.stats["bytes_sent_per_sec"]) / 1024
                    },
                    "download": {
                        "avg": sum(self.stats["bytes_recv_per_sec"]) / len(self.stats["bytes_recv_per_sec"]) / 1024,
                        "max": max(self.stats["bytes_recv_per_sec"]) / 1024,
                        "min": min(self.stats["bytes_recv_per_sec"]) / 1024
                    }
                },
                "packets": {
                    "sent": sum(self.stats["packets_sent_per_sec"]) / len(self.stats["packets_sent_per_sec"]),
                    "received": sum(self.stats["packets_recv_per_sec"]) / len(self.stats["packets_recv_per_sec"])
                },
                "errors": {
                    "in": max(self.stats["errors_in"]),
                    "out": max(self.stats["errors_out"])
                },
                "drops": {
                    "in": max(self.stats["drops_in"]),
                    "out": max(self.stats["drops_out"])
                }
            }
        }
        
        return report
        
    def run_profile(self):
        """Run the network profiling"""
        try:
            print(f"Starting network profile for interface {self.interface_name}")
            print(f"IP Address: {self.interface_ip}")
            print("-" * 50)
            
            # Collect metrics
            self.collect_metrics()
            
            # Generate and print report
            print("\n\nNetwork Profile Report")
            print("-" * 50)
            report = self.generate_report()
            
            print("\nInterface Information:")
            print(json.dumps(report["interface_info"], indent=2))
            
            print("\nPerformance Metrics:")
            metrics = report["metrics"]
            print(f"Duration: {metrics['duration']} seconds")
            print(f"Samples: {metrics['samples']}")
            print("\nThroughput (KB/s):")
            print(f"  Upload: Avg={metrics['throughput']['upload']['avg']:.2f}, "
                  f"Max={metrics['throughput']['upload']['max']:.2f}, "
                  f"Min={metrics['throughput']['upload']['min']:.2f}")
            print(f"  Download: Avg={metrics['throughput']['download']['avg']:.2f}, "
                  f"Max={metrics['throughput']['download']['max']:.2f}, "
                  f"Min={metrics['throughput']['download']['min']:.2f}")
            print("\nPacket Rate (packets/sec):")
            print(f"  Sent: {metrics['packets']['sent']:.2f}")
            print(f"  Received: {metrics['packets']['received']:.2f}")
            print("\nErrors:")
            print(f"  In: {metrics['errors']['in']}")
            print(f"  Out: {metrics['errors']['out']}")
            print("\nDrops:")
            print(f"  In: {metrics['drops']['in']}")
            print(f"  Out: {metrics['drops']['out']}")
            
        except KeyboardInterrupt:
            print("\nProfiling stopped by user")
        except Exception as e:
            print(f"\nError during profiling: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Network Interface Profiler")
    parser.add_argument("ip", help="Interface IP to profile")
    parser.add_argument("--duration", type=int, default=60,
                      help="Profile duration in seconds")
    args = parser.parse_args()
    
    profiler = NetworkProfiler(args.ip, args.duration)
    profiler.run_profile()
